// use 'var' instead of 'let' here so that popup can access background page's variables
var LOGIN_STATE_KEY = 'is_logged_in';
var is_logged_in = false;
var SERVER_BASE_URL = 'https://dynalist.io/';
var UPDATE_LOGIN_INTERVAL = 10000;
var TITLE_FORMAT = 'title-format';
var NOTE_FORMAT = 'note-format';
var TITLE_FORMAT_WITH_SELECTION = 'title-format-with-selection';
var NOTE_FORMAT_WITH_SELECTION = 'note-format-with-selection';

let login_state_change_listener = null;

function register_login_state_change_listener(callback) {
	login_state_change_listener = callback;
}

function update_is_logged_in_state(new_value, invoke_callback = true) {
	if (is_logged_in !== new_value && invoke_callback) {
		login_state_change_listener && login_state_change_listener();
	}

	is_logged_in = new_value;
}

function attempt_update_login(invoke_callback = true) {
	let request = new XMLHttpRequest();
	request.open('POST', SERVER_BASE_URL + 'api/user/session', true);
	request.setRequestHeader('Content-Type', 'text/plain; charset=UTF-8');
	request.onload = function () {
		if (request.status === 200) {
			let parsed_response = JSON.parse(request.responseText);
			let new_is_logged_in = parsed_response.session.signed_in;

			chrome.storage.sync.set(
				{[LOGIN_STATE_KEY]: new_is_logged_in},
				() => {
					update_is_logged_in_state(new_is_logged_in, invoke_callback);
				}
			);
		}
		else if (request.status !== 200) {
			chrome.storage.sync.set(LOGIN_STATE_KEY, false, () => {
				update_is_logged_in_state(false, invoke_callback);
			});
		}
	};
	request.send({});
}

function capture(item_title, note, success_callback, failure_callback) {
	post('api/doc/inbox', {
		content: item_title,
		note: note
	}, success_callback, failure_callback);
}

function get_formatted_item_and_note(selected_text, page_title, page_url, callback) {
	chrome.storage.sync.get({
		[TITLE_FORMAT]: '%title%',
		[NOTE_FORMAT]: '%url%',
		[TITLE_FORMAT_WITH_SELECTION]: '%selected%',
		[NOTE_FORMAT_WITH_SELECTION]: 'From [%title%](%url%)',
	}, (items) => {
		let title_format = items[TITLE_FORMAT];
		let note_format = items[NOTE_FORMAT];
		let title_format_with_selection = items[TITLE_FORMAT_WITH_SELECTION];
		let note_format_with_selection = items[NOTE_FORMAT_WITH_SELECTION];

		let title_to_send = '';
		let note_to_send = '';

		if (selected_text) {
			title_to_send = process_format(title_format_with_selection, selected_text, page_title, page_url);
			note_to_send = process_format(note_format_with_selection, selected_text, page_title, page_url);
		}
		else {
			title_to_send = process_format(title_format, selected_text, page_title, page_url);
			note_to_send = process_format(note_format, selected_text, page_title, page_url);
		}

		callback(title_to_send, note_to_send);
	});
}

function post(url, data, success_callback, failure_callback) {
	let request = new XMLHttpRequest();
	request.onreadystatechange = () => {
		if (request.readyState === 4) {
			if (request.status === 200) {
				success_callback && success_callback(request.responseText);
			}
			else {
				failure_callback && failure_callback(request.responseText);
			}
		}
	};
	request.open('POST', SERVER_BASE_URL + url, true);
	request.setRequestHeader('Content-Type', 'text/plain; charset=UTF-8');
	request.send(JSON.stringify(data));
}

function capture_from_context_menu(info, tab) {
	let selected_text = '';
	let page_title = '';
	let page_url = '';

	if (info.selectionText) {
		selected_text = info.selectionText;
	}
	else if (info.linkUrl) {
		selected_text = info.linkUrl;
	}

	if (tab.title) {
		page_title = tab.title;
	}
	if (info.pageUrl) {
		page_url = info.pageUrl;
	}

	get_formatted_item_and_note(selected_text, page_title, page_url, capture);
}

function process_format(format, selected_text, page_title, url) {
	return format.replace('%selected%', selected_text).replace(/%title%/g, page_title).replace(/%url%/g, url);
}

function setup_context_menu_items() {
	chrome.contextMenus.create({
		title: "Clip current page URL to Dynalist",
		onclick: capture_from_context_menu
	});

	chrome.contextMenus.create({
		title: "Clip selected text Dynalist",
		contexts: ["selection"],
		onclick: capture_from_context_menu
	});

	chrome.contextMenus.create({
		title: "Clip this link to Dynalist",
		contexts: ["link"],
		onclick: capture_from_context_menu
	});
}

setup_context_menu_items();
attempt_update_login();

