let login_screen_el = document.querySelectorAll('.login-screen')[0];
let capture_screen_el = document.querySelectorAll('.capture-screen')[0];
let capture_success_screen_el = document.querySelectorAll('.capture-success-screen')[0];
let capture_failure_screen_el = document.querySelectorAll('.capture-failure-screen')[0];

let bg_page = chrome.extension.getBackgroundPage();
let is_mac_like = /(Mac|iPhone|iPod|iPad)/i.test(navigator.platform);

update_login_state_in_ui();

// Always hide these screens when opening the popup
hide(capture_success_screen_el);
hide(capture_failure_screen_el);

bg_page.register_login_state_change_listener(update_login_state_in_ui);

function update_login_state_in_ui() {
	if (!bg_page.is_logged_in) {
		bg_page.attempt_update_login();
		hide(capture_screen_el);
		show(login_screen_el);
	}
	else {
		hide(login_screen_el);
		show(capture_screen_el);
	}
}

function ready(fn) {
	if (document.attachEvent ? document.readyState === 'complete' : document.readyState !== 'loading') {
		fn();
	}
	else {
		document.addEventListener('DOMContentLoaded', fn);
	}
}

function open_login_page() {
	chrome.tabs.create({url: bg_page.SERVER_BASE_URL + 'login'});

	window.close();
}

function capture_to_inbox() {
	let item_title_el = document.querySelectorAll('.item-title')[0];
	let item_note_el = document.querySelectorAll('.item-note')[0];

	bg_page.capture(item_title_el.value, item_note_el.value, (response) => {
		let parsed_response = JSON.parse(response);

		if (parsed_response._code === 'Ok') {
			hide(capture_screen_el);
			show(capture_success_screen_el);

			setTimeout(close_popup, 1000);
		}
		else {
			let fail_reason_el = document.querySelectorAll('.capture-fail-reason')[0];
			if (parsed_response._msg) {
				fail_reason_el.textContent = parsed_response._msg;
			}
			else {
				fail_reason_el.textContent = '';
			}
			error_callback();

			bg_page.attempt_update_login(false);
		}
	}, (response) => {
		error_callback();
	});
}

function error_callback() {
	hide(capture_screen_el);
	show(capture_failure_screen_el);
}

function close_popup() {
	window.close();
}

function on_document_load() {
	// attach login listener
	let login_link_el = document.querySelectorAll('.js-login')[0];
	login_link_el.removeEventListener('click', open_login_page);
	login_link_el.addEventListener('click', open_login_page);

	// attach send listener
	let send_link_el = document.querySelectorAll('.js-send')[0];
	send_link_el.removeEventListener('click', capture_to_inbox);
	send_link_el.addEventListener('click', capture_to_inbox);

	// attach cancel listener
	let cancel_link_els = document.querySelectorAll('.js-close-popup');

	for (let cancel_link_el of cancel_link_els) {
		cancel_link_el.removeEventListener('click', close_popup);
		cancel_link_el.addEventListener('click', close_popup)
	}

	document.addEventListener('keydown', function(event) {
		if (event.key === 'Enter') {
			if ((is_mac_like && event.metaKey) || (!is_mac_like && event.ctrlKey)) {
				event.preventDefault();
				capture_to_inbox();
			}
		}
	});
}

function show(el) {
	el.style.display = 'block';
}

function hide(el) {
	el.style.display = 'none';
}

chrome.tabs.query({active: true, currentWindow: true}, function (arrayOfTabs) {
	// since only one tab should be active and in the current window at once
	// the return variable should only have one entry
	let item_title_el = document.querySelectorAll('.item-title')[0];
	let item_note_el = document.querySelectorAll('.item-note')[0];
	let activeTab = arrayOfTabs[0];

	chrome.tabs.executeScript({
		code: 'window.getSelection().toString();'
	}, function (selection) {
		let selected_text = selection[0];
		let page_title  = activeTab.title;
		let page_url = activeTab.url;

		bg_page.get_formatted_item_and_note(selected_text, page_title, page_url, (processed_title, processed_note) => {
			item_title_el.value = processed_title;
			item_note_el.value = processed_note;
		})
	});
});

ready(on_document_load);
